/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.core.artifact.annotation.ArtifactSupport;

public class Artifacts {

    private static final Logger log = LoggerFactory.getLogger(Artifacts.class);

    public static final String DEFAULT_CLASSIFIER = "";

    protected static final Map<String, Class<?>> supportClasses = new HashMap<>();

    static {
        Artifacts.registerSupport(Blueprint.class);
    }


    public static final void registerSupport(Class<?> supportClass) {
        if (supportClass.isAnnotationPresent(ArtifactSupport.class)) {
            String artifactType = supportClass.getAnnotation(ArtifactSupport.class).supportedCategory();
            Artifacts.supportClasses.put(artifactType, supportClass);
        }
    }

    public static final DefaultArtifact create(String category) {
        if (supportClasses.containsKey(category)) {
            Class<?> supportClass = supportClasses.get(category);
            try {
                Constructor<?> c = supportClass.getConstructor();
                Object instance = c.newInstance();
                if (DefaultArtifact.class.isAssignableFrom(instance.getClass()))
                    return (DefaultArtifact) instance;
            } catch (NoSuchMethodException e) {
                log.error("Error creating artifact", e);
            } catch (SecurityException e) {
                log.error("Error creating artifact", e);
            } catch (InstantiationException e) {
                log.error("Error creating artifact", e);
            } catch (IllegalAccessException e) {
                log.error("Error creating artifact", e);
            } catch (IllegalArgumentException e) {
                log.error("Error creating artifact", e);
            } catch (InvocationTargetException e) {
                log.error("Error creating artifact", e);
            }
        }
        return new DefaultArtifact();
    }

    public static final DefaultArtifact fromGenerated(Generated generated) {
        DefaultArtifact result = Artifacts.create(generated.getType());
        if (result != null)
            result.setContent(generated.getObject());
        return result;
    }

    public static final DefaultArtifact fromFile(File source, String fileRoot) {
        return fromFile(source, fileRoot, null);
    }

    public static final DefaultArtifact fromFile(File source, String fileRoot, String category) {
        String ownPath = source.getAbsolutePath().replace('\\', '/').substring(fileRoot.length());
        if (ownPath.startsWith("/"))
            ownPath = ownPath.substring(1);
        log.debug("Artifacts.fromFile: \n\tFILE: [{}]\n\tPATH: [{}]", source.getAbsolutePath(), ownPath);

        String currentPath = ownPath;

        int index = currentPath.lastIndexOf('/');
        if (index < 0) {
            log.error("Filename not found in path [{}]", ownPath);
            return null;
        }
        String fileName = currentPath.substring(index + 1);
        currentPath = currentPath.substring(0, index);

        index = currentPath.lastIndexOf('/');
        if (index < 0) {
            log.error("version not found in path [{}]", ownPath);
            return null;
        }
        String version = currentPath.substring(index + 1);
        currentPath = currentPath.substring(0, index);

        index = currentPath.lastIndexOf('/');
        if (index < 0) {
            log.error("artifactId not found in path [{}]", ownPath);
            return null;
        }
        String artifactId = currentPath.substring(index + 1);
        currentPath = currentPath.substring(0, index);

        String groupId = currentPath.replaceAll("/", ".");

        index = fileName.lastIndexOf('.');
        if (index < 0) {
            log.error("extension not found in path [{}]", ownPath);
            return null;
        }
        String extension = fileName.substring(index + 1);

        String classifier = category;
        if (!CommonUtils.isValid(category)) {
            String fileNamePrefix = artifactId + "-" + version;
            index = fileName.indexOf(fileNamePrefix);
            if (index < 0)
                return null;
            classifier = fileName.substring(index + fileNamePrefix.length() + 1, fileName.lastIndexOf('.'));
        }

        if (!CommonUtils.isValid(classifier))
            classifier = DEFAULT_CLASSIFIER;

        log.debug("Artifacts.fromFile:" +
                "\n\tCOORDINATES:\n\tgroupId: [{}]\n\tartifactId: [{}]\n\tversion: [{}]\n\textension: [{}]\n\tclassifier: [{}]",
                groupId,
                artifactId,
                version,
                extension,
                classifier);

        if (!CommonUtils.isValid(groupId) || !CommonUtils.isValid(artifactId) || !CommonUtils.isValid(version)
                || !CommonUtils.isValid(extension))
            return null;

        DefaultArtifact result = create(classifier);

        if (result != null) {
            result.getCoordinates().groupId(groupId).artifactId(artifactId).version(version).type(extension)
                    .classifier(classifier);
            byte[] contents;
            try {
                contents = Files.readAllBytes(source.toPath());
                result.setContent(contents);
            } catch (IOException e) {
                log.error("Error setting artifact contents", e);
                return null;
            }
        }

        return result;
    }

}
